/*
 * CKEditor - The text editor for Internet - http://ckeditor.com
 * Copyright (C) 2003-2008 Frederico Caldeira Knabben
 *
 * == BEGIN LICENSE ==
 *
 * Licensed under the terms of any of the following licenses at your
 * choice:
 *
 *  - GNU General Public License Version 2 or later (the "GPL")
 *    http://www.gnu.org/licenses/gpl.html
 *
 *  - GNU Lesser General Public License Version 2.1 or later (the "LGPL")
 *    http://www.gnu.org/licenses/lgpl.html
 *
 *  - Mozilla Public License Version 1.1 or later (the "MPL")
 *    http://www.mozilla.org/MPL/MPL-1.1.html
 *
 * == END LICENSE ==
 */

CKEDITOR.dialog.add( 'link', function( editor )
{
	// Handles the event when the "Target" selection box is changed.
	var targetChanged = function()
	{
		var popupFeatures = this.getDialog().getContentElement( 'target', 'popupFeatures' ).getElement(),
			targetName = this.getDialog().getContentElement( 'target', 'linkTargetName' ),
			value = this.getValue();
		if ( value == 'popup' )
		{
			popupFeatures.show();
			targetName.setLabel( editor.lang.linkTargetPopupName );
		}
		else
		{
			popupFeatures.hide();
			targetName.setLabel( editor.lang.linkTargetFrameName );
			this.getDialog().setValueOf( 'target', 'linkTargetName', value.charAt( 0 ) == '_' ? value : '' );
		}
	},
	// Handles the event when the "Type" selection box is changed.
		linkTypeChanged = function()
	{
		var dialog = this.getDialog(),
			partIds = [ 'urlOptions', 'anchorOptions', 'emailOptions' ],
			typeValue = this.getValue();
		if ( typeValue == 'url' )
		{
			dialog.showPage( 'target' );
			dialog.showPage( 'upload' );
		}
		else
		{
			dialog.hidePage( 'target' );
			dialog.hidePage( 'upload' );
		}

		for ( var i = 0 ; i < partIds.length ; i++ )
		{
			var element = dialog.getContentElement( 'info', partIds[i] ).getElement().getParent().getParent();
			if ( partIds[i] == typeValue + 'Options' )
				element.show();
			else
				element.hide();
		}
	};

	return {
		title : editor.lang.linkTitle,
		minWidth : 400,
		minHeight : 320,
		contents : [
			{
				id : 'info',
				label : editor.lang.linkInfo,
				title : editor.lang.linkInfo,
				elements :
				[
					{
						id : 'linkType',
						type : 'select',
						label : editor.lang.linkType,
						'default' : 'url',
						items :
						[
							[ editor.lang.url, 'url' ],
							[ editor.lang.linkToAnchor, 'anchor' ],
							[ editor.lang.linkToEmail, 'email' ]
						],
						onChange : linkTypeChanged
					},
					{
						type : 'vbox',
						id : 'urlOptions',
						children :
						[
							{
								type : 'hbox',
								widths : [ '25%', '75%' ],
								children :
								[
									{
										id : 'protocol',
										type : 'select',
										label : editor.lang.protocol,
										'default' : 'http://',
										style : 'width : 100%;',
										items :
										[
											[ 'http://' ],
											[ 'https//' ],
											[ 'ftp://' ],
											[ 'news://' ],
											[ '<other>', '' ]
										]
									},
									{
										type : 'text',
										id : 'url',
										label : editor.lang.url,
										validate : function()
										{
											if ( this.getDialog().getValueOf( 'info', 'linkType' ) != 'url' )
												return true;

											var func = CKEDITOR.dialog.validate.notEmpty( editor.lang.linkNoUrl );
											return func.apply( this );
										}
									}
								]
							},
							{
								type : 'button',
								id : 'browse',
								label : editor.lang.browseServer
							}
						]
					},
					{
						type : 'vbox',
						id : 'anchorOptions',
						width : 260,
						align : 'center',
						padding : 0,
						children :
						[
							{
								type : 'html',
								html : CKEDITOR.tools.htmlEncode( editor.lang.linkSelectAnchor )
							},
							{
								type : 'hbox',
								children :
								[
									{
										type : 'select',
										id : 'anchorName',
										'default' : '',
										label : editor.lang.linkAnchorName,
										style : 'width: 100%;',
										items :
										[
											[ '' ]
										]
									},
									{
										type : 'select',
										id : 'anchorId',
										'default' : '',
										label : editor.lang.linkAnchorId,
										style : 'width: 100%;',
										items :
										[
											[ '' ]
										]
									}
								]
							}
						]
					},
					{
						type :  'vbox',
						id : 'emailOptions',
						padding : 1,
						children :
						[
							{
								type : 'text',
								id : 'emailAddress',
								label : editor.lang.linkEmailAddress,
								validate : function()
								{
									if ( this.getDialog().getValueOf( 'info', 'linkType' ) != 'email' )
										return true;

									var func = CKEDITOR.dialog.validate.notEmpty( editor.lang.linkNoEmail );
									return func.apply( this );
								}
							},
							{
								type : 'text',
								id : 'emailSubject',
								label : editor.lang.linkEmailSubject
							},
							{
								type : 'textarea',
								id : 'emailBody',
								label : editor.lang.linkEmailBody,
								rows : 3,
								'default' : ''
							}
						]
					}
				]
			},
			{
				id : 'target',
				label : editor.lang.linkTarget,
				title : editor.lang.linkTarget,
				elements :
				[
					{
						type : 'hbox',
						widths : [ '50%', '50%' ],
						children :
						[
							{
								type : 'select',
								id : 'linkTargetType',
								label : editor.lang.linkTarget,
								'default' : 'notSet',
								style : 'width : 100%;',
								'items' :
								[
									[ editor.lang.linkTargetNotSet, 'notSet' ],
									[ editor.lang.linkTargetFrame, 'frame' ],
									[ editor.lang.linkTargetPopup, 'popup' ],
									[ editor.lang.linkTargetNew, '_blank' ],
									[ editor.lang.linkTargetTop, '_top' ],
									[ editor.lang.linkTargetSelf, '_self' ],
									[ editor.lang.linkTargetParent, '_parent' ]
								],
								onChange : targetChanged
							},
							{
								type : 'text',
								id : 'linkTargetName',
								label : editor.lang.linkTargetFrameName
							}
						]
					},
					{
						type : 'vbox',
						width : 260,
						align : 'center',
						padding : 2,
						id : 'popupFeatures',
						children :
						[
							{
								type : 'html',
								html : CKEDITOR.tools.htmlEncode( editor.lang.linkPopupFeatures )
							},
							{
								type : 'hbox',
								children :
								[
									{
										type : 'checkbox',
										id : 'resizable',
										label : editor.lang.linkPopupResizable
									},
									{
										type : 'checkbox',
										id : 'status',
										label : editor.lang.linkPopupStatusBar
									}
								]
							},
							{
								type : 'hbox',
								children :
								[
									{
										type : 'checkbox',
										id : 'location',
										label : editor.lang.linkPopupLocationBar
									},
									{
										type : 'checkbox',
										id : 'toolbar',
										label : editor.lang.linkPopupToolbar
									}
								]
							},
							{
								type : 'hbox',
								children :
								[
									{
										type : 'checkbox',
										id : 'menubar',
										label : editor.lang.linkPopupMenuBar
									},
									{
										type : 'checkbox',
										id : 'fullscreen',
										label : editor.lang.linkPopupFullScreen
									}
								]
							},
							{
								type : 'hbox',
								children :
								[
									{
										type : 'checkbox',
										id : 'scrollbars',
										label : editor.lang.linkPopupScrollBars
									},
									{
										type : 'checkbox',
										id : 'dependent',
										label : editor.lang.linkPopupDependent
									}
								]
							},
							{
								type : 'hbox',
								children :
								[
									{
										type :  'text',
										widths : [ '30%', '70%' ],
										labelLayout : 'horizontal',
										label : editor.lang.linkPopupWidth,
										id : 'popupWidth'
									},
									{
										type :  'text',
										labelLayout : 'horizontal',
										widths : [ '55%', '45%' ],
										label : editor.lang.linkPopupLeft,
										id : 'popupLeft'
									}
								]
							},
							{
								type : 'hbox',
								children :
								[
									{
										type :  'text',
										labelLayout : 'horizontal',
										widths : [ '30%', '70%' ],
										label : editor.lang.linkPopupHeight,
										id : 'popupHeight'
									},
									{
										type :  'text',
										labelLayout : 'horizontal',
										label : editor.lang.linkPopupTop,
										widths : [ '55%', '45%' ],
										id : 'popupTop'
									}
								]
							}
						]
					}
				]
			},
			{
				id : 'upload',
				label : editor.lang.linkUpload,
				title : editor.lang.linkUpload,
				elements :
				[
					{
						type : 'file',
						id : 'upload',
						label : editor.lang.upload,
						action : 'nowhere.php',
						size : 38
					},
					{
						type : 'fileButton',
						id : 'uploadButton',
						label : editor.lang.uploadSubmit,
						'for' : [ 'upload', 'upload' ]
					}
				]
			},
			{
				id : 'advanced',
				label : editor.lang.linkAdvanced,
				title : editor.lang.linkAdvanced,
				elements :
				[
					{
						type : 'vbox',
						padding : 1,
						children :
						[
							{
								type : 'hbox',
								widths : [ '45%', '35%', '20%' ],
								children :
								[
									{
										type : 'text',
										id : 'advId',
										label : editor.lang.linkId
									},
									{
										type : 'select',
										id : 'advLangDir',
										label : editor.lang.linkLangDir,
										'default' : '',
										style : 'width: 100%;',
										items :
										[
											[ editor.lang.linkLangDirNotSet, '' ],
											[ editor.lang.linkLangDirLTR, 'ltr' ],
											[ editor.lang.linkLangDirRTL, 'rtl' ]
										]
									},
									{
										type : 'text',
										id : 'advAccessKey',
										label : editor.lang.linkAcccessKey,
										maxLength : 1
									}
								]
							},
							{
								type : 'hbox',
								widths : [ '45%', '35%', '20%' ],
								children :
								[
									{
										type : 'text',
										label : editor.lang.linkName,
										id : 'advName'
									},
									{
										type : 'text',
										label : editor.lang.linkLangCode,
										id : 'advLangCode'
									},
									{
										type : 'text',
										label : editor.lang.linkTabIndex,
										id : 'advTabIndex',
										maxLength : 5
									}
								]
							}
						]
					},
					{
						type : 'vbox',
						padding : 1,
						children :
						[
							{
								type : 'hbox',
								widths : [ '45%', '55%' ],
								children :
								[
									{
										type : 'text',
										label : editor.lang.linkAdvisoryTitle,
										id : 'advTitle'
									},
									{
										type : 'text',
										label : editor.lang.linkAdvisoryContentType,
										id : 'advContentType'
									}
								]
							},
							{
								type : 'hbox',
								widths : [ '45%', '55%' ],
								children :
								[
									{
										type : 'text',
										label : editor.lang.linkCSSClasses,
										id : 'advCSSClasses'
									},
									{
										type : 'text',
										label : editor.lang.linkCharset,
										id : 'advCharset'
									}
								]
							},
							{
								type : 'hbox',
								children :
								[
									{
										type : 'text',
										label : editor.lang.linkStyles,
										id : 'advStyles'
									}
								]
							}
						]
					}
				]
			}
		],
		onShow : function()
		{
			this.getContentElement( 'info', 'url' ).focus();
		},
		onOk : function()
		{
			var attributes = { href : 'javascript:void(0)/*' + CKEDITOR.tools.getNextNumber() + '*/' },
				linkList, subject, body, me = this;

			// Compose the URL.
			switch ( this.getValueOf( 'info', 'linkType' ) )
			{
				case 'url':
					attributes._cksavedhref = this.getValueOf( 'info', 'protocol' ) + this.getValueOf( 'info', 'url' );
					break;
				case 'anchor':
					attributes._cksavedhref = '#' + ( this.getValueOf( 'info', 'anchorName' ) || this.getValueOf( 'info', 'anchorId' ) );
					break;
				case 'email':
					linkList = [ 'mailto:', this.getValueOf( 'info', 'emailAddress' ) ];
					subject = encodeURICompoenent( this.getValueOf( 'info', 'emailSubject' ) );
					body = encodeURIComponent( this.getValueOf( 'info', 'emailBody' ) );
					if ( subject || body)
					{
						linkList.push( '?' );
						subject && linkList.push( 'subject=', subject );
						body && body.push( 'body=', body );
					}
					attributes._cksavedhref = linkList.join( '' );
					break;
				default:
			}
			
			// Popups and target.
			if ( this.getValueOf( 'target', 'linkTargetType' ) == 'popup' )
			{
				var onclickList = [ 'window.open(this.href, \'',
						this.getValueOf( 'target', 'linkTargetName' ), '\', \'' ],
					featureList = [ 'resizable', 'status', 'location', 'toolbar', 'menubar', 'fullscreen',
						'scrollbars', 'dependent' ],
					featureLength = featureList.length,
					addFeature = function( inputName, featureName )
					{
						if ( me.getValueOf( 'target', inputName ) != '' )
							featureList.push( featureName + '=' + me.getValueOf( 'target', inputName ) );
					};

				for ( var i = 0 ; i < featureLength ; i++ )
					featureList[i] = featureList[i] + ( this.getValueOf( 'target', featureList[i] ) ? '=yes' : '=no' );
				addFeature( 'popupWidth', 'width' );
				addFeature( 'popupLeft', 'left' );
				addFeature( 'popupHeight', 'height' );
				addFeature( 'popupTop', 'top' );

				onclickList.push( featureList.join( ',' ), '\'); return false;' );
				attributes._cksavedonclick = onclickList.join( '' );
			}
			else
			{
				if ( this.getValueOf( 'target', 'linkTargetType' ) != 'notSet' )
					attributes.target = this.getValueOf( 'target', 'linkTargetName' );
			}

			// Advanced attributes.
			var advAttr = function( inputName, attrName )
			{
				var value = me.getValueOf( 'advanced', inputName );
				if ( value != '' )
					attributes[attrName] = value;
			};
			advAttr( 'advLangDir', 'dir' );
			advAttr( 'advAccessKey', 'accessKey' );
			advAttr( 'advName', 'name' );
			advAttr( 'advLangCode', 'lang' );
			advAttr( 'advTabIndex', 'tabindex' );
			advAttr( 'advTitle', 'title' );
			advAttr( 'advContentType', 'type' );
			advAttr( 'advCSSClasses', 'class' );
			advAttr( 'advCharset', 'charset' );
			advAttr( 'advStyles', 'style' );

			// Create element if current selection is collapsed, else apply style.
			var style = new CKEDITOR.style( { element : 'a', attributes : attributes } );
			style.type = CKEDITOR.STYLE_INLINE;		// need to override... dunno why.
			style.apply( this.getParentEditor().document );
			
			// Id. Apply only to the first link.
			if ( this.getValueOf( 'advanced', 'advId' ) != '' )
			{
				var links = this.getParentEditor().document.$.getElementsByTagName( 'a' );
				for ( var i = 0 ; i < links.length ; i++ )
				{
					if ( links[i].href == attributes.href )
					{
						console.log( 'fadhm' );
						links[i].id = this.getValueOf( 'advanced', 'advId' );
						break;
					}
				}
			}

			// style.apply() changed the selection already, so clear the old one.
			this.clearSavedSelection();
		}
	};
} );
